import cherrypy
import cgi
import util

class Wrapper(object):
    """
    Wraps up dictionaries, internal class. Lets you use getattr
    just like getitem.
    """
    def __init__(self, wrapped=None):
        if wrapped:
            self._d = wrapped
        else:
            self._d = {}
    def __delitem__(self, key):
        del self._d[key]
    def __getattribute__(self, key):
        try:
            return object.__getattribute__(self, key)
        except AttributeError:
            raise
        try:
            return self[key]
        except KeyError, e:
            raise AttributeError, e
    def __getitem__(self, key):
        return self._d[key]
    def __setitem__(self, key, value):
        self._d[key] = value
    def __setattr__(self, key, value):
        if key[0] == "_":
            object.__setattr__(self, key, value)
        else:
            self._d[key] = value
    def clear(self):
        return self._d.clear()
    def update(self, d):
        return self._d.update(d)
    def get(self, *args, **kwargs):
        return self._d.get(*args, **kwargs)
    def keys(self):
        return self._d.keys()
    def items(self):
        return self._d.items()
    def values(self):
        return self._d.values()
    def has_key(self, k):
        return self._d.has_key(k)
    def __contains__(self, k):
        return k in self._d
    def __repr__(self):
        return self._d.__repr__()
    def __str__(self):
        return self._d.__str__()
    def __len__(self):
        return len(self._d)

class Session(Wrapper):
    """
    A more user-friendly session object that allows the
    end user to use session.keyname instead of session["keyname"],
    though the latter still works.
    """
    d = property(lambda self: cherrypy.session)
    def __init__(self):
        Wrapper.__init__(self)

class Request(object):
    """
    The Subway request object.
    
    Properties:
    args - Wrapper dictionary of the request arguments (cherrypy.request.paramMap)
    headers - Wrapper dictionary of the incoming headers (cherrypy.request.headerMap)
    cookies - Wrapper dictionary of the incoming HTTP cookies (cherrypy.request.simpleCookie)
    path - the request URL path
    url - the URL the browser used to fetch this resource
    method - the HTTP method used to fetch this resource
    """
    args = property(lambda self: Wrapper(cherrypy.request.paramMap))
    headers = property(lambda self: Wrapper(cherrypy.request.headerMap))
    cookies = property(lambda self: cherrypy.request.simpleCookie)
    path = property(lambda self: cherrypy.request.path)
    url = property(lambda self: cherrypy.request.browserUrl)
    method = property(lambda self: cherrypy.request.method)

class Response(object):
    """
    The Subway response object.

    Properties:
    headers - Wrapper dictionary of the outgoing headers (cherrypy.response.headerMap)
    cookies - Wrapper dictionary of the outgoing cookies (cherrypy.response.simpleCookie)

    Methods:
    write(content) - write out content to the browser
    redirect(url) - redirect the user to a given URL. you should return this return value
                    from your function.
    escape(content) - HTML escape content and return it
    """
    headers = property(lambda self: Wrapper(cherrypy.response.headerMap))
    cookies = property(lambda self: cherrypy.response.simpleCookie)
    def write(self, d):
        cherrypy.response.body.append(d)
    def redirect(self, url):
        raise cherrypy.HTTPRedirect(url)
    def escape(self, msg):
        return util.escape(msg)
    def no_cache(self):
        cherrypy.response.headerMap["Expires"] = "Sat, 6 May 1995 12:00:00 GMT"
        #cherrypy.response.headerMap["Cache-Control"] = "no-store, no-cache, must-revalidate"
        #Cache-control: post-check=0, pre-check=0
        cherrypy.response.headerMap["Cache-Control"] = "no-store, no-cache, must-revalidate, post-check=0, pre-check=0"
        cherrypy.response.headerMap["Pragma"] = "no-cache"

