import cherrypy
from cherrypy.lib.filter import basefilter
#from cherrypy.lib import httptools
from urllib import quote

class AuthConfigError(Exception):
    """
    A problem with the AuthFilter has occurred.
    """
    pass

class AccessDeniedError(Exception):
    """
    The user was denied access.
    """
    pass

_USER_TOKEN = '_user'

def get_user():
    """
    Get the user object, None if user isn't logged in.
    """
    
    return cherrypy.session.get(_USER_TOKEN, None)
        

def login(user):
    """
    Specify a user has logged in.
    
    The parameter is a user object. This can be anything,
    such as a custom object (an "avatar") containing
    user information, or just a string containing the username.
    """
    
    cherrypy.session[_USER_TOKEN] = user

def logout():
    """
    Specify a user has logged out.
    """    
    
    if _USER_TOKEN in cherrypy.session:        
        del cherrypy.session[_USER_TOKEN]


MESSAGE_NOT_LOGGED_IN = "You are not logged in."
MESSAGE_DOT_NOT_HAVE_ROLE = "You do not have the %(role)r role."

class AuthFilter(basefilter.BaseFilter):
    
    def beforeMain(self):
        
        # is the filter turned on?
        if not cherrypy.config.get('subway.auth.on', False):
            return
        
        # first, make sure he's logged in.
        user = cherrypy.session.get(_USER_TOKEN, None)
        if not user:
            message = cherrypy.config.get("subway.auth.not_logged_in_message", MESSAGE_NOT_LOGGED_IN)
        
            #@@ HACK: temp hack to make it to work
            redirect_url = cherrypy.config.get("subway.auth.redirect_url", None)                
            raise cherrypy.HTTPRedirect(redirect_url.replace("$url", quote(cherrypy.request.browserUrl)).replace("$message", str("Not logged in")))

            #raise AccessDeniedError(message)
        
        # next, if there are required_roles, check them
        required_roles = cherrypy.config.get("subway.auth.required_roles", None)
        if required_roles:            
            # there are required roles, check them            
            try:
                roles = user.roles
            except AttributeError:
                raise AuthConfigError("User object did not have a 'roles' attribute, and the requested resource required roles.")
            
            # intersect the two role lists and see what remains
            if set(user.roles) & set(required_roles):
                # ok, user has at least one required role
                pass
            else:
                # we use a dict here, so you are free of not to provide a role placeholder in your message
                message = cherrypy.config.get("subway.auth.no_role_message", MESSAGE_DOT_NOT_HAVE_ROLE) % dict(role=role)
                #raise AccessDeniedError(message)
                
                # this is called when the user is not authorized
                redirect_url = cherrypy.config.get("subway.auth.redirect_url", None)                
                raise cherrypy.HTTPRedirect(redirect_url.replace("$url", quote(cherrypy.request.browserUrl)).replace("$message", str(e)))
            
            #~ for role in required_roles:
                #~ # user does not have a given role
                #~ if role not in roles: 
                    #~ message = cherrypy.config.get("subway.auth.no_role_message", MESSAGE_DOT_NOT_HAVE_ROLE) % {"role" : role}
                    #~ raise AccessDeniedError(message)

    def beforeErrorResponse(self):
        
        # is the filter turned on?
        if not cherrypy.config.get('subway.auth.on', False):
            return
        
        try:
            # raise last exception
            raise
        except AccessDeniedError, e:
            self.onAccessDeniedError(e)
        #~ except: #@@ needed?
            #~ raise
            
    def onAccessDeniedError(self, e):
        # this is called when the user is not authorized
        redirect_url = cherrypy.config.get("subway.auth.redirect_url", None)
        if redirect_url:
            # redirect to the URL; we can't raise HTTPRedirect at this point, so use httptools
            # 
            # the login function may want to know what the message and referer URL were, 
            # so provide them via special variables        
            #httptools.redirect()  #@@ httptools is deprecated
            raise cherrypy.HTTPRedirect(redirect_url.replace("$url", quote(cherrypy.request.browserUrl)).replace("$message", str(e)))
        else:            
            # if the user did not specify a redirect URL, just re-raise
            raise 


#~ class RoleUser(object):
    #~ """
    #~ Base class you can extend for role-based access control.
    #~ """
    
    #~ def __init__(self, roles):
        #~ """
        #~ Roles is a list of roles the user has
        #~ """
        #~ self.roles = roles


