import common

SEND_AND_WAIT = 0

def chunked_flow(method):
    """
    Decorator which denotes that a method is a CHUNKD flow method. You must explicitly
    add the exposed decorator in addition to this one for security reasons.

    You must yield SEND_AND_WAIT in order to send the data.
    """
    flowid = common._get_flowid()
    def _m(self, _resume=0, *args, **kwargs):
        # this is the actual cherrypy handler function
        # _resume is the magic variable which lets the server know
        # at which point in the flow the client is resuming.
        _resume = int(_resume)
        gen = common._get_gen(flowid, _resume, method, self)
        # the next loop iterates thru a generator, yielding its data until encountering SEND_AND_WAIT
        for result in gen:
            if result == SEND_AND_WAIT:
                break
            else:
                yield result
        common._save_resumepoint(flowid, gen)
    return _m
