"""
See docstring for flow decorator.
"""

import cherrypy

import gencopy
import gentree


_counter = 0 # this is the seed variable which assigns each flow a unique ID; internal

back_button_error = not gencopy.can_copy_generators # should we not allow the back button or refreshes?

MILESTONE = -1

def save_milestone(flowid, resumeid):
    cherrypy.session["flow_%d_milestone" % flowid] = resumeid

# the following internal functions encompass most of the common
# functionality between the two different decorators (flow and chunked_flow)

def _get_flowid():
    """
    Internal.
    """
    # first, give this flow an id and increment the counter
    global _counter
    flowid = _counter
    _counter += 1
    return flowid

def _check_resume(flowid, _resume):
    """
    Internal.
    """
    global back_button_error
    if _resume != 0 and cherrypy.session.get("flow_%d_milestone", 0) > _resume:
        raise cherrypy.InternalError, "You cannot go back to before the last milestone."
    # if we can't copy generators, we should make sure the client doesn't abuse the back button and corrupt data
    # because previous resume point state will be corrupted.
    if _resume != cherrypy.session.get("flow_%d_lastresume" % flowid, 0) and _resume != 0 and back_button_error:
        #print "Session Map: %s" % cherrypy.sessions.cherryFlowSession.get("flow_%d_lastresume" % flowid, 0)
        #print "_resume: %s" % _resume
        #print "error: %s" % back_button_error
        # if we are not on the current resumepoint, and we aren't starting the flow again, and we should display errors...
        raise cherrypy.InternalError, "Please don't use the back button or refresh." # display the error
    cherrypy.session["flow_%d_lastresume" % flowid] = cherrypy.request.resume_id

def _save_resumepoint(flowid, gen):
    """
    Internal.
    """
    cherrypy.session["flow_%d_%d" % (flowid, cherrypy.request.resume_id)] = gen # save the current resume point...

def _get_gen(flowid, _resume, method):
    """
    Internal.
    """
    cherrypy.request.resume_id = _resume + 1 # generate the resume id for the current resume point
    _check_resume(flowid, _resume)
    # first, try to fetch the saved state from the last resume point
    gen = cherrypy.session.get("flow_%d_%d" % (flowid, _resume), None)
    if not gen or _resume == 0:
        gen = gentree.GeneratorTree(method()) # if there was no saved state, start the flow anew
    return gencopy.copy(gen) # copy the generator so the previous resume point's state remains untouched

    
