"""
This module exposes a copy method which can copy generators.
"""


import copy as defaultcopy
import types

# this block analyzes the runtime environment to determine
# if it can copy running generators.

def defcopy(obj):
    """
    Default copier which just uses copy.copy, and will just skip over generators.
    """
    if isinstance(obj, types.GeneratorType):
        return obj
    else:
        return defaultcopy.copy(obj)

copy = None

can_copy_generators = False

try:
    def foo():
        yield 5
    defaultcopy.copy(foo()) # try to copy using system-default copy (copy is initialized to it)
    copy = defaultcopy.copy
    can_copy_generators = True
    # if we get to this point, defaultcopy.copy worked ok.
except TypeError:
    try: # we can't copy using defaultcopy.copy; try using statesaver.
        import statesaver
        # if we get to this point, statesaver is installed and should work OK.
        copy = statesaver.copy
        can_copy_generators = True
    except ImportError:
        import warnings
        copy = defcopy
        can_copy_generators = False
        warnings.warn("Unable to copy generators. Please install Stackless Python (http://www.stackless.com/) or statesaver (source available at http://codespeak.net/svn/user/arigo/misc/statesaver.c, binaries at http://bub-n-bros.sourceforge.net/) to allow the back button to work.")
        