"""
I am a module which treats nested generators as one big flat generator.
"""

import gencopy

import new
import types

class GeneratorTree:
    """
    I am an internal utility class which wraps a running generator. It allows a generator which yields other generators
    to be treated as a flat generator.
    """
    def __init__(self, g):
        self.stack = [g]
    def __copy__(self):
        # when copied, be sure to use gencopy.
        c = new.instance(self.__class__)
        c.stack = map(gencopy.copy, self.stack)
        return c
    def __iter__(self):
        return self
    def inst_build(self):
        return new.instance(self.__class__)
    def next(self):
        if len(self.stack) == 0: # no more generators left
            raise StopIteration
        try:
            result = self.stack[-1].next() # call the most recently called generator
        except StopIteration: # end of this one, call the next one in the parent
            self.stack.pop() # this one is done
            return self.next() # run it again
        if isinstance(result, types.GeneratorType): # if the result is a generator...
            self.stack.append(result) # add it as the most recent generator
            return self.next() # call it
        else:
            return result # otherwise its a normal result
        
