from cherrypy.lib import cptools
from cherrypy import _cputil
import cherrypy

import sys
import traceback

import util, view

def expose(m):
    return view.docstring_view(cherrypy.expose(m))

class Controller(cptools.PositionalParametersAware):
    """
    Base Controller class.
    """
    def __init__(self, module):
        self._module = module      
    
    def __getattribute__(self, k):
        try:
            return object.__getattribute__(self, k)
        except AttributeError:
            return getattr(self._module, k)
            
    def _cpOnError(self):
        try:
            # Re-raise last exception
            raise            
        except cherrypy.NotFound, exc: # handle "not found" condition
            exc_info = sys.exc_info()            
            try:
                handler = _cputil.getSpecialAttribute("_error_404")
                if getattr(handler, 'exposed') and handler.exposed:
                    cherrypy.response.body = [handler(exc)]                    
                else:
                    #@@ TODO: debugging hint to expose _error_404?                    
                    #cherrypy.log('_error_404 handler was found but it's not exposed')
                    raise cherrypy.InternalError
                    
            except cherrypy.InternalError:
                
                # couldn't find the _error_404, or it wasn't exposed.
                #@@ TODO: debugging hint to expose _error_404?                
                cherrypy.response.status = '404 Not Found'
                cherrypy.response.headerMap["Content-type"] = "text/html"
    
                type, value, tb = exc_info
                
                body = HTML_404 % dict(
                    exc=''.join(map(util.escape, traceback.format_exception_only(type, value))),
                    tb=''.join(map(util.escape, traceback.format_exception(type, value, tb)))                
                )                
                
                cherrypy.response.body = [body]
            
                #@@ TODO: log exception using cherry.log('')

        except Exception, exc: # handle everything else            
            exc_info = sys.exc_info()
            try:
                handler = _cputil.getSpecialAttribute("_error_500")
                if getattr(handler, 'exposed') and handler.exposed:
                    cherrypy.response.body = [handler(exc)]
                else:
                    #@@ TODO: debugging hint to expose _error_404?                    
                    #cherrypy.log('_error_500 handler was found but it's not exposed')
                    raise cherrypy.InternalError
                    
            except cherrypy.InternalError:
                
                # couldn't find _error_500, or it wasn't exposed.
                cherrypy.response.status = '500 Internal Server Error'
                cherrypy.response.headerMap["Content-type"] = "text/html"

                type, value, tb = exc_info
                
                body = HTML_500 % dict(
                    exc=''.join(map(util.escape, traceback.format_exception_only(type, value))),
                    tb=''.join(map(util.escape, traceback.format_exception(type, value, tb)))                
                )                
                                
                #@@ TODO: log exception using cherry.log()
                
                cherrypy.response.body = [body]

# Internal Server Error
HTML_500 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Internal Server Error (500)</title>
<style type="text/css">@import url('/public/error.css');</style>  
</head>
<body>     

<h1>Internal Server Error <span>(code 500)</span></h1>
<p>A sofware problem occurred while processing your request.</p>

<h2>Description</h2>    
<p>%(exc)s</p>
    
<h2>Traceback</h2>
<pre>
%(tb)s
</pre>    

</body>
</html>
"""

# Not Found
HTML_404 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Not Found (404)</title>
<style type="text/css">@import url('/public/error.css');</style>  
</head>
<body>     

<h1>Not Found <span>(code 404)</span></h1>
<p>A sofware problem occurred while processing your request.</p>

<h2>Description</h2>    
<p>%(exc)s</p>
    
<h2>Traceback</h2>
<pre>
%(tb)s
</pre>    

</body>
</html>
"""

