import imp
import os
import sys
import inspect

import namespaces

import subway

def magic_import(code, filename, namespace):
    modname = os.path.splitext(os.path.basename(filename))[0]
    mod = imp.new_module(modname)
    mod.__file__ = filename
    sys.modules[modname] = mod
    for (k,v) in namespace.items():
        setattr(mod, k, v)
    code = compile(code, filename, "exec")
    exec code in mod.__dict__
    return mod

def import_with_superglobals(filename):
    return magic_import(open(filename, "r").read(), filename, namespaces.get_superglobal_namespace())

def import_controller(filename, view_dir, model):
    base_view_dir,view_dir_name = os.path.split(view_dir)
    try:
        viewobj = None
        viewobj = subway.ViewManager(base_view_dir)
        viewobj = getattr(viewobj, view_dir_name)
    except AttributeError:
        # no views defined for this controller
        pass
    mod = magic_import(open(filename, "r").read(),
                        filename,
                        namespaces.get_controller_namespace(
                            viewobj,
                            model))
    return subway.Controller(mod)

def import_models(filenames):
    model = imp.new_module("model")
    sys.modules["model"] = model
    for filename in filenames:
        mod = magic_import(open(filename, "r").read(),
                            filename,
                            namespaces.get_model_namespace())
        sys.modules["model." + os.path.basename(filename)] = mod
        for (k,v) in inspect.getmembers(mod):
            if getattr(v, "_subway_model", False) and v is not subway.Model:
                setattr(model, k, v)
    return model

def import_daemon(filename, model):
    mod = magic_import(open(filename, "r").read(),
                        filename,
                        namespaces.get_daemon_namespace(model))
    assert callable(getattr(mod, "main", None)), "Every daemon must have a callable main() function"
    return mod.main
    